module Appfuel
  module Service
    module Config
      # Defines the configuration for the whole worker. It basically has two
      # child definitions :db and :sneakers
      #
      # Configuration Overview
      #
      # env:        The environement this worker is deployed as dev, qa, stg, prod
      # logfile:    The location of the application log file if the value is
      #             stdout or stderr it will use that instead
      # dispatchers List of workers that dispatch messages to actions. The reason
      #             why you would want more than one is to use different exchange
      #             types
      #
      # db          Database configuration please see sp_offers/config/database
      # sneakers    RabbitMQ configuration please see sp_offers/config/sneakers
      # aws         Configuration S3 where we store our documents
      # @return Definition
      def self.worker_definition
        Appfuel::Config.define :worker do
          file 'config/app.yaml'
          defaults log_file:       'stdout',
                    log_level:     'info',
                    audit_logfile: 'stdout'

          validator {
            required(:env).filled(:str?)
            required(:log_file).filled(:str?)
            required(:log_level).filled(:str?)
            required(:audit_logfile).filled(:str?)
            # Children will be validated on there own
            # we are just ensuring they exist
            optional(:db).filled(:hash?)
            required(:sneakers).filled(:hash?)
          }
          self << [
            Config.sneakers_definition,
            Appfuel::Config.db_definition,
          ]
        end
      end
    end
  end
end
